//----------------------------------------------------------------------------
// SVG Viewer - Version 1.0
// A library of SVG renderer
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
// 
// Contains elements from:
//			AGG - a graphic library (Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
// Uses functions from:
//          Expat - a XML parser library (Copyright James Clark)
// 
// SVG Viewer is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------

#ifndef SVG_ATTR_PARSER_INCLUDED
#define SVG_ATTR_PARSER_INCLUDED
#include "svg_basics.h"
#include "agg_svg_path_tokenizer.h"

namespace svg
{

//////////////////////////////////////////////////////////////////////
// parser: used by SVG graph nodes (the element class instances)
// to load them and their data. Initailized by the root element of SVG graph
//////////////////////////////////////////////////////////////////////
class attr_parser  
{
    public:
		//Initialization: give the references to storages, that have to be filled
		//with the SVG graph data

		attr_parser(path_storage &path, attribute_storage &attr);

        ~attr_parser();

    public:
        // XML event handlers
        void parse(element &elm, const char** attr);

        handle parse_attr(const char* name, const char* value);

		//////////////////////////////////////////////////////////////////////
		// Specialized, helper functions for complicated attributes:
		//////////////////////////////////////////////////////////////////////

		//parse the "d" attribute - path coordinates and stores it into a path_storage
		//Arguments:
		//	str:	the value of the svg::d xml attribute - geometry of the path		
		//the handle, returned by this function, is a handle to a path, not to an attribute!
        handle parse_d(const char* str);
		
		//parse the "points" attribute - polygon/polyline coordinates and stores it into a path_storage
		//Arguments:
		//	str:		the value of the svg::points xml attribute - vertices of the polygon/polyline		
		//	close_flag:	true, if the element is svg:polygon, and false for the svg:polyline		
		//the handle, returned by this function, is a handle to a path, not to an attribute!
        handle parse_points(const char* str, bool close_flag);
		
		//default handler for parsing the style expression
		//Arguments: 
		//	str:	the value of the xml svg::style attribute
		//It recursively calls the parse_attr() for every style item
        handle parse_style(const char* str);

		//default handler for parsing the transformation expression
		//Arguments: 
		//	str:	the value of the xml svg::transform attribute
		//It creates at least 2, or at most 6 subsequent attributes
        handle parse_transform(const char* str);

	private:        
		//helper functions for parsing the transformation
        handle parse_matrix(const char* str);
        handle parse_translate(const char* str);
        handle parse_rotate(const char* str);
        handle parse_scale(const char* str);
        handle parse_skew_x(const char* str);
        handle parse_skew_y(const char* str);

		//helper functions for parsing the style
        bool parse_name_value(const char* nv_start, const char* nv_end);
        void copy_name(const char* start, const char* end);
        void copy_value(const char* start, const char* end);
        
    protected:
		//References to path (geometry) and attribute storages
		//(they are presisted in the root element of SVG graph)
		path_storage	&m_path;
		attr_storage	&m_attr;

}; //attr_parser


}; //namespace svg
#endif SVG_ATTR_PARSER_INCLUDED
