//----------------------------------------------------------------------------
// svgtex - a SVG vector texture plugin for Blender
// Copyright (C) 2008 Witold Jaworski
// Contact:	witold.jaworski@airplanes3d.net
//			http://airplanes3d.net
//
// This program is based on:	
//			vectex - a SVG vector texture plugin for Blender
//			(Copyright (C) 2008 Marcelo de Gomensoro Malheiros, 
//			 mgmalheiros@gmail.com)
// 
// Contains elements from:
//			AGG - a graphic library 
//			(Copyright Maxim Shemanarev, mcseem@antigrain.com)
//          http://antigrain.com
//
// Uses functions from:
//          Expat - a XML parser library 
//			(Copyright James Clark)
// 
// svgtext is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// SVG Viewer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with SVG Viewer; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
// MA 02110-1301, USA.
//----------------------------------------------------------------------------
#ifndef SVGTEX_INCLUDED
#define SVGTEX_INCLUDED
#include "svg_basics.h"

#ifdef __cplusplus
#define EXT extern "C"
#else
#define EXT
#endif

#ifdef DEBUG
#define MSG0(F)         printf(F);
#define MSG1(F,A)       printf(F, (A));
#define MSG2(F,A,B)     printf(F, (A), (B));
#define MSG3(F,A,B,C)   printf(F, (A), (B), (C));
#define MSG4(F,A,B,C,D) printf(F, (A), (B), (C), (D));
#else
#define MSG0(F)
#define MSG1(F,A)
#define MSG2(F,A,B)
#define MSG3(F,A,B,C)
#define MSG4(F,A,B,C,D)
#endif

#define TEX (6<<9) //the input text field is not defined in plugin.h!

typedef svg::rgba8 color;
typedef svg::pixfmt pixfmt;
typedef agg::int8u int8u;

//General constants
enum constants
{
	file_name_size	= 64,
	bytes_pp		= svg::pixfmt::pix_width, //bytes per pixel
	max_fltr_range  = 10,					  //max. range of the averaging (pixels)
	tile_level		= 8,
	tile_size		= 1 << tile_level, //256
	tile_border		= (max_fltr_range/2) + (max_fltr_range%2) + 1
};

// Data structure, used in communication with Blender 
// Cast must have max size of 32 floats = 128 bytes and all fields be 32-bit aligned 
// The array varstr in svgtex.cpp module and the Cast structure must describe the  
// same fields, in the same order !
typedef struct Cast
{

	int           interpol;
	unsigned char base_color[4];
	int		      def_LOD;
    int		      mem_max;
    int           label_dummy0;
    int           enabled;

    float         fidelity;
    float         alpha;
    int           fix_def_LOD;
    int           mem;
    int           label_dummy1;
    char          file_name[file_name_size];

    void         *instance_data;
	//-------- helper methods ----------
	color bkgnd_color()
	{
		color result(base_color[0],base_color[1],base_color[2]);
		if (alpha < 1.0) result.opacity(alpha);
		return result;
	}

} Cast;

/*------ declarations of functions called back by Blender -----*/
void plugin_init(void);
void plugin_callback(int);
void plugin_instance_init(Cast *);
int  plugin_tex_doit(int, Cast *, float *, float *, float *, float *);
#endif //SVGTEX_INCLUDED

